package de.lmu.eipnf.javapong;

/**
 * @author Michael Kirsch
 * @Version 0.0.2
 * Diese Klasse repräsentiert das Spielfeld und enthält alle Elemente und die Steuerung der Padels.
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

public class Spielfeld extends JPanel implements KeyListener, Runnable { // Spielfeld ist ein JPanel und implementiert das Interface Keylistener

    int ballXpos; // Ballposition X-Achse
    int ballYpos; // Ballposition Y-Achse

    int paddelRechtsYpos; // Paddelposition Rechts Y-Achse
    int paddelLinksYpos; // Paddelposition Links Y-Achse

    boolean paddelLinksHochBewegen; // Zustand von linken Paddel; true = in Bewegeung, false = nicht in Bewegung
    boolean paddelLinksRunterBewegen; // Zustand von rechtem Paddel; true = in Bewegeung, false = nicht in Bewegung
    boolean paddelRechtsHochBewegen; // Zustand von rechtem Paddel; true = in Bewegeung, false = nicht in Bewegung
    boolean paddelRechtsRunterBewegen; // Zustand von rechtem Paddel; true = in Bewegeung, false = nicht in Bewegung

    boolean gameIsOn;


    public Spielfeld() { // Konstruktor; Bisher ohne weitere Funktion
        ballXpos = 100;
        ballYpos = 10;
        paddelLinksYpos = 250;
        paddelRechtsYpos = 250;
        gameIsOn = true;
        new Thread(this).start(); //Starte den Animations-Thread --> run-Methode
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);
        Graphics2D painter = (Graphics2D) g;
        painter.setColor(Color.red);
        painter.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        painter.fillOval(ballXpos, ballYpos, 20, 20);

        painter.setColor(Color.CYAN);
        painter.setColor(Color.GRAY);

        int paddelSchrittweite = 4;

        if (paddelLinksHochBewegen == true) {
            if (paddelLinksYpos > 0) {
                paddelLinksYpos = paddelLinksYpos - paddelSchrittweite;
            }
        }

        if (paddelLinksRunterBewegen == true) {
            if (paddelLinksYpos < 500) {
                paddelLinksYpos = paddelLinksYpos + paddelSchrittweite;
            }
        }

        if (paddelRechtsHochBewegen == true) {
            if (paddelRechtsYpos > 0) {
                paddelRechtsYpos = paddelRechtsYpos - paddelSchrittweite;
            }
        }

        if (paddelRechtsRunterBewegen == true) {
            if (paddelRechtsYpos < 500) {
                paddelRechtsYpos = paddelRechtsYpos + paddelSchrittweite;
            }
        }


        painter.fillRect(0, paddelLinksYpos, 20, 100);
        painter.fillRect(580, paddelRechtsYpos, 20, 100);


    }

    @Override
    public void keyTyped(KeyEvent e) { // Wenn Taste gedrückt und wieder losgelassen wurde

    }

    @Override
    public void keyPressed(KeyEvent e) { // Wenn eine Taste gedrückt

        if (e.getKeyChar() == 'w') {
            paddelLinksHochBewegen = true;
        }

        if (e.getKeyChar() == 's') {
            paddelLinksRunterBewegen = true;
        }

        if (e.getKeyCode() == KeyEvent.VK_DOWN) { // Wenn die Taste == untere Pfeiltaste
            paddelRechtsRunterBewegen = true;
        }

        if (e.getKeyCode() == KeyEvent.VK_UP) { // Wenn die Taste == obere Pfeiltaste
            paddelRechtsHochBewegen = true;
        }

    }

    @Override
    public void keyReleased(KeyEvent e) { // Wenn Taste losgelassen wurde

        if (e.getKeyChar() == 'w') {
            paddelLinksHochBewegen = false;
        }

        if (e.getKeyChar() == 's') {
            paddelLinksRunterBewegen = false;
        }

        if (e.getKeyCode() == KeyEvent.VK_DOWN) { // Wenn die Taste == untere Pfeiltaste
            paddelRechtsRunterBewegen = false;
        }

        if (e.getKeyCode() == KeyEvent.VK_UP) { // Wenn die Taste == obere Pfeiltaste
            paddelRechtsHochBewegen = false;
        }

    }

    // Innerhalb der Run-Methode wird die Animation gesteuert.
    @Override
    //noinspection InfiniteLoopStatement
    public void run() {
        int schrittweiteXRichtung = 1; //Schrittweite des Balls
        int schrittweiteYRichtung = 1; //Schrittweite des Balls

        while (gameIsOn) {

            try {

                // ToDo: Code zum Paddel Grenzen hier zusätzlich abfragen

                // Aktualisiere die Position des Balls um den Offset
                if (ballXpos == 580) {
                    schrittweiteXRichtung = schrittweiteXRichtung * -1;
                }
                if (ballYpos == 580) {

                    schrittweiteYRichtung = schrittweiteYRichtung * -1;
                }
                if (ballXpos == 0) {
                    schrittweiteXRichtung = schrittweiteXRichtung * -1;
                }

                if (ballYpos == 0) {
                    schrittweiteYRichtung = schrittweiteYRichtung * -1;
                }

                ballXpos = ballXpos + schrittweiteXRichtung;
                ballYpos = ballYpos + schrittweiteYRichtung;
                repaint(); //Aktualsisiere das Bild

                Thread.currentThread().sleep(5); // Lege den Thread für fünf Millisekunden schlafen

            } catch (InterruptedException e) { // Fehlerbehandlung
                e.printStackTrace();
            }

        }

    }
}
